/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: windowtofront.c,v 1.5 2023/04/26 18:51:49 jacadcaps Exp $
 
	Desc: Move window in front of all other windows
	Lang: english
*/
#include <proto/layers.h>
#include "intuition_intern.h"
#include "inputhandler_actions.h"
#include "inputhandler_support.h"
#include "mosmisc.h"

struct WindowToFrontActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
};

static VOID int_windowtofront(struct WindowToFrontActionMsg *msg,
							  struct IntuitionBase *IntuitionBase);

/*****************************************************************************
 
	NAME */
#include <intuition/intuition.h>
#include <proto/intuition.h>

AROS_LH1(void, WindowToFront,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 52, Intuition)

/*  FUNCTION
	Bring a window to the front (ie. before any other window).
 
	INPUTS
	window - Which window
 
	RESULT
	None.
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
	29-10-95    digulla automatically created from
			    intuition_lib.fd and clib/intuition_protos.h
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct WindowToFrontActionMsg msg;

	DEBUG_WINDOWTOFRONT(dprintf("WindowToFront: Window 0x%lx\n", window));

	if (!window)
		return;

	msg.window = window;
	DoASyncAction((APTR)int_windowtofront, &msg.msg, sizeof(msg), IntuitionBase);

	AROS_LIBFUNC_EXIT

} /* WindowToFront */


static VOID int_windowtofront(struct WindowToFrontActionMsg *msg,
							  struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;
	struct Layer *layer = WLAYER(window);
	struct Screen *screen = window->WScreen;
	struct Requester *req;

	DEBUG_WINDOWTOFRONT(dprintf("IntWindowToFront: Window 0x%lx\n", window));

	if (!(layer->Flags & LAYERBACKDROP))
	{
		LockLayers(&screen->LayerInfo);
		//LOCK_REFRESH(screen);

		/* GZZ or regular window? */
		if (BLAYER(window))
		{
			/* bring outer window to front first!! */

			UpfrontLayer(0, BLAYER(window));
		}

		UpfrontLayer(0, layer);

		for (req = window->FirstRequest; req; req = req->OlderRequest)
		{
			if (req->ReqLayer)
			{
				MoveLayerInFrontOf(req->ReqLayer, layer);
			}
		}

		#ifdef SKINS
		ArrangeChildren(window,IntuitionBase);
		#endif

		if ((IS(window->WScreen)->SpecialFlags & SF_AppearingBar) && !MENUS_ACTIVE)
		{
			UpfrontLayer(0,window->WScreen->BarLayer);
		}

		CheckLayers(screen, IntuitionBase);

		//UNLOCK_REFRESH(screen);
		UnlockLayers(&screen->LayerInfo);

	}

	NotifyDepthArrangement(window, IntuitionBase);

	UpdateScreenPointer(screen,IntuitionBase);

	if (((struct IntWindow *)(window))->specialflags & SPFLAG_ICONIFIED)
		ShowWindow(window);
	/* window is not iconified anymore */
}
